interface IReadonlyCollection<T> {
    /**
     * Count the element in the collection.
     * @getter
     */
    readonly size: number;
    /**
     * Iterable.
     */
    [Symbol.iterator](): IterableIterator<T>;
}
interface ICollection<T> extends IReadonlyCollection<T> {
    /**
     * Release memory.
     */
    destroy(): void;
    /**
     * Remove all elements.
     * Notice that this method does not release memory.
     */
    clear(): void;
}

/**
 * Adjacent list
 */
type IAdjacentList = number[][];
interface IDigraphEdge {
    /**
     * Target point of the edge.
     */
    to: number;
}
interface IDigraph<E extends IDigraphEdge = IDigraphEdge> {
    /**
     * The number of nodes in the graph. (0-index)
     */
    N: number;
    /**
     * Adjacency list. G[i] represent the index list of the edges start from node i.
     */
    G: number[][];
    /**
     *
     */
    edges: E[];
}

type ICompare<T> = (x: T, y: T) => -1 | 0 | 1 | number;
type IEquals<T> = (x: T, y: T) => boolean;
/**
 * Make all properties in `T` mutable.
 * @see https://stackoverflow.com/a/46634877
 */
type Mutable<T extends object> = {
    -readonly [P in keyof T]: T[P];
};
/**
 * Make a set of properties by key `K` become optional from `T`.
 */
type PickPartial<T extends object, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
/**
 * Make an object deep-readonly.
 */
type DeepReadonly<T extends object> = {
    readonly [P in keyof T]: T[P] extends object ? DeepReadonly<T[P]> : Readonly<T[P]>;
};

interface IOperator<T> {
    readonly ZERO: T;
    readonly ONE: T;
    add(x: T, y: T): T;
    subtract(x: T, y: T): T;
    divide(x: T, y: T): T;
    multiple(x: T, y: T): T;
}

export { DeepReadonly, IAdjacentList, ICollection, ICompare, IDigraph, IDigraphEdge, IEquals, IOperator, IReadonlyCollection, Mutable, PickPartial };
